package gitpb

import (
	"errors"
	"os"
	"path/filepath"
	"strings"

	"go.wit.com/log"
)

// returns true if 'git pull' will work
func (repo *Repo) IsBranchRemote(branchname string) bool {
	if branchname == "" {
		return false
	}
	if repo.Exists(filepath.Join(".git/refs/remotes/origin", branchname)) {
		// todo: actually use .git/config
		return true
	}
	return false
}

// returns true if 'git pull' will work
func (repo *Repo) ExistsUserBranchRemote() bool {
	branchname := repo.GetUserBranchName()
	if repo.IsBranchRemote(branchname) {
		return true
	}
	return false
}

// returns true if the user branch exists
func (repo *Repo) ExistsUserBranch() bool {
	if repo.GetUserBranchName() == "" {
		return false
	}
	branchname := repo.GetUserBranchName()
	if repo.Exists(filepath.Join(".git/refs/heads", branchname)) {
		// todo: actually use .git/config
		return true
	}
	return false
}

// returns true if the devel branch exists
func (repo *Repo) ExistsDevelBranch() bool {
	if repo.GetDevelBranchName() == "" {
		return false
	}
	branchname := repo.GetDevelBranchName()
	if repo.Exists(filepath.Join(".git/refs/heads", branchname)) {
		// todo: actually use .git/config
		return true
	}
	return false
}

func (repo *Repo) GetBranchHash(branchname string) string {
	if branchname == "" {
		return ""
	}
	if repo.Exists(filepath.Join(".git/refs/remotes", branchname)) {
		return readRefHash(filepath.Join(repo.FullPath, ".git/refs/remotes", branchname))
	}
	if repo.Exists(filepath.Join(".git/refs/heads", branchname)) {
		return readRefHash(filepath.Join(repo.FullPath, ".git/refs/heads", branchname))
	}
	return ""
}

func (repo *Repo) GetBranchVersion(branchname string) string {
	if branchname == repo.GetUserBranchName() {
		return "user"
	}
	if branchname == repo.GetDevelBranchName() {
		return "devel"
	}
	if branchname == repo.GetMasterBranchName() {
		return "master"
	}
	base, branchname := filepath.Split(branchname)
	if base != "" {
		if branchname == repo.GetUserBranchName() {
			return "remote user"
		}
		if branchname == repo.GetDevelBranchName() {
			return "remote devel"
		}
		if branchname == repo.GetMasterBranchName() {
			return "remote master"
		}
	}
	return ""
}

func readRefHash(filename string) string {
	data, _ := os.ReadFile(filename)
	return string(data)
}

func (repo *Repo) GetLocalBranches() []string {
	return ListFiles(filepath.Join(repo.GetFullPath(), "/.git/refs/heads"))
}

func (repo *Repo) GetRemoteBranches() []string {
	remotes := ListFiles(filepath.Join(repo.GetFullPath(), "/.git/refs/remotes"))
	return remotes
}

// git describe --tags e548b0fb6d0d14cdfb693850d592419f247dc2b1
// v0.22.61-15-gbab84d7
func (repo *Repo) GetHashName(h string) (string, error) {
	h = strings.TrimSpace(h)
	log.Info("GetHashName() is looking for", repo.GetGoPath(), h)
	cmd := []string{"git", "describe", "--tags", h}
	r, err := repo.RunStrict(cmd)
	if err != nil {
		return "", err
	}
	if len(r.Stdout) == 0 {
		return "", errors.New("git describe was empty")
	}
	return r.Stdout[0], nil
}

// lookup a hash from a tag with 'git rev-list'
func (repo *Repo) GetTagHash(t string) string {
	// git rev-list -n 1 v0.0.66
	cmd := []string{"git", "rev-list", "-n", "1", t}
	result, _ := repo.RunStrict(cmd)
	// log.Info("getLastTagVersion()", result.Stdout)

	if len(result.Stdout) == 0 {
		// log.Log(WARN, "no gitpb.LastTag() repo is broken. ignore this.", repo.GetGoPath())
		return ""
	}

	return result.Stdout[0]
}
